<?php
define("MIFARE_LOCK_FILENAME", dirname(__FILE__) . "/mifare_lock.~~~");

function mifare_acquire_lock() {
  global $mifare_lock_handle, $mifare_shutdown_eval;

  if (file_exists(MIFARE_LOCK_FILENAME)) {
    if (!@unlink(MIFARE_LOCK_FILENAME))
      return false;
  }
  $mifare_lock_handle = fopen(MIFARE_LOCK_FILENAME, "w");

  $mifare_shutdown_eval = 'fclose($GLOBALS["mifare_lock_handle"]);unlink(MIFARE_LOCK_FILENAME);return true;';
  register_shutdown_function("assert", 'eval($GLOBALS["mifare_shutdown_eval"])');

  return true;
}

function mifare_init_library() {
  global $mifare_shutdown_eval;

  $imports = array('z2mInitialize', 'z2mFinalize', 'z2mFindFirstReaderA', 'z2mCloseFindReader', 'z2mOpenReaderA',
                   'z2mCloseReader', 'z2mFindCard', 'z2mOpenCard', 'z2mReadCardData', 'z2mWriteULCardPage', 'z2mCloseCard');

  if (!function_exists("wb_load_library"))
    return false;
  $z2mlib = wb_load_library("Z2m.dll");
  if (!$z2mlib)
    return false;

  $mifare_shutdown_eval = 'wb_release_library(' . $z2mlib . ');' . $mifare_shutdown_eval;

  foreach ($imports as $import) {
    if (! $GLOBALS[$import] = wb_get_function_address($import, $z2mlib))
      return false;
  }

  global $z2mInitialize, $z2mFinalize;
  $result = wb_call_function($z2mInitialize);
  if ($result != 0)
    return false;
  $mifare_shutdown_eval = 'wb_call_function(' . $z2mFinalize . ');' . $mifare_shutdown_eval;
  return true;
}

function mifare_open_reader() {
  global $z2mFindFirstReaderA, $z2mCloseFindReader, $z2mOpenReaderA, $z2mCloseReader, $mifare_shutdown_eval;

  $VHandle = pack("V", 0);
  $szPort = str_repeat("\0", 16);
  $VInfo = pack("VVVVVVV", wb_get_address($szPort), 16, 0, 0, 0, 0, 0);
  $result = wb_call_function($z2mFindFirstReaderA, array($VHandle, 0, $VInfo, 0));
  if ($result != 0)
    return false;
  list(, $VHandle) = unpack("V*", $VHandle);
  wb_call_function($z2mCloseFindReader, array($VHandle));
  list(, , , $nTip, $nSN) = unpack("V*", $VInfo);

  $VHandle = pack("V", 0);
  $result = wb_call_function($z2mOpenReaderA, array($VHandle, $szPort, $nSN, $nTip, 0 , 0));
  if ($result != 0)
    return false;
  list(, $VHandle) = unpack("V*", $VHandle);

  $mifare_shutdown_eval = 'wb_call_function(' . $z2mCloseReader . ', array(' . $VHandle . '));' . $mifare_shutdown_eval;

  global $mfReaderHandle;
  $mfReaderHandle = $VHandle;

  return true;
}

function mifare_find_card() {
  global $mfReaderHandle, $z2mFindCard;

  $VInfo = str_repeat("\0", 16) . pack("V", 0);
  $result = wb_call_function($z2mFindCard, array($mfReaderHandle, $VInfo, 0));
  if ($result != 0)
    return false;
  $num = substr($VInfo, 1, ord($VInfo[0]));
  list(, $type) = unpack("V*", substr($VInfo, 16));
  if ($type != 1)
    return false;

  return $num;
}

function mifare_read_card($cardId, $addr, $len) {
  global $mfReaderHandle, $z2mOpenCard, $z2mReadCardData, $z2mCloseCard;

  if (!$cardId)
    return false;

  $VHandle = pack("V", 0);
  $ACdNum = str_pad(chr(strlen($cardId)) . $cardId, 16, "\0");
  $result = wb_call_function($z2mOpenCard, array($VHandle, $mfReaderHandle, 1, $ACdNum, 0));
  if ($result != 0)
    return false;
  list(, $VHandle) = unpack("V*", $VHandle);

  $VBuf = str_repeat("\0", 64);
  $VRead = pack("V", 0);
  $result = wb_call_function($z2mReadCardData, array($VHandle, 0, $VBuf, 64, $VRead, 0));
  if ($result != 0) {
    wb_call_function($z2mCloseCard, array($VHandle));
    return false;
  }
  list(, $VRead) = unpack("V*", $VRead);

  wb_call_function($z2mCloseCard, array($VHandle));

  if ($VRead != 64)
    return false;

  return substr($VBuf, $addr, $len);
}

function mifare_write_card($cardId, $addr, $writeData) {
  global $mfReaderHandle, $z2mOpenCard, $z2mWriteULCardPage, $z2mCloseCard;

  if (!$cardId)
    return false;

  $VHandle = pack("V", 0);
  $ACdNum = str_pad(chr(strlen($cardId)) . $cardId, 16, "\0");
  $result = wb_call_function($z2mOpenCard, array($VHandle, $mfReaderHandle, 1, $ACdNum, 0));
  if ($result != 0)
    return false;
  list(, $VHandle) = unpack("V*", $VHandle);

  $writeData = str_pad($writeData, ceil(strlen($writeData) / 4) * 4);
  $pageNo = floor($addr / 4);
  for ($i = 0; $i < strlen($writeData); $pageNo++, $i += 4) {
    $result = wb_call_function($z2mWriteULCardPage, array($VHandle, $pageNo, substr($writeData, $i, 4), 0));
    if ($result != 0) {
      wb_call_function($z2mCloseCard, array($VHandle));
      return false;
    }
  }

/*
  $VWrite = pack("V", 0);
  $result = wb_call_function($z2mWriteCardData, array($VHandle, $addr, $writeData, strlen($writeData), $VWrite, 0));
  if ($result != 0) {
    wb_call_function($z2mCloseCard, array($VHandle));
    return false;
  }
  list(, $VWrite) = unpack("V*", $VWrite);
*/

  wb_call_function($z2mCloseCard, array($VHandle));

  return true;
}
?>